pc.script.create("GlobalScript", function (context) {
 
    var GlobalScript = function (entity) {
        // Cache the entity that this script instance affects
        this.entity = entity;
    };
    
	var copyEntity = function (e) {
		
		var newEntity = new pc.fw.Entity();
			
		newEntity.setLocalPosition(e.getLocalPosition());
		newEntity.setLocalRotation(e.getLocalRotation());
		newEntity.setLocalScale(e.getLocalScale());
		
		var systems = context.systems.list();
		for (var s = 0; s < systems.length; s++) {
			var key = systems[s].name;
			if (e[key]) {
				context.systems[key].addComponent(newEntity, e[key].data);
			}
		}

		for (var i = 0; i < e.getChildren().length; i++) {
            newEntity.addChild(copyEntity(e.getChildren()[i]));
        }
				
		return newEntity;
	};
	
	var createBox = function () {
		
		var entity = new pc.fw.Entity();
 
		// Create a new Primitive Component and add it to the Entity.
		context.systems.primitive.addComponent(entity, {
			type: 'Box',
			color: '0xff0000',
		});

		// Add to the Hierarchy
		context.root.addChild(entity);
		
		return entity;
	};
	
	var NeedInit = true;
	var World;
	var sizeX = 0;
	var sizeY = 0;
	
	function random(seed) {
	  if (!this.seed)
		this.seed = 1984;//new Date().getTime();
	  this.seed = (this.seed*9301+49297) % 233280;
	  return this.seed/(233280.0);
	}
 
    GlobalScript.prototype = {
		initialize: function () {
		},
		
        update: function (dt) {
			if(NeedInit)
			{
				var scene = pc.scene.Scene.current;
				if(typeof scene != 'undefined')
				{
					NeedInit = false;
					
					var MaxLightNum = 5;
					pc.scene.Scene.MaxLightNum = MaxLightNum;
			
					var LightDeclaration = "";
					var LightCumulate = "";
										
					for(i = 0;i < MaxLightNum;i++) {	
						LightDeclaration += "uniform vec4 LightPos" + i + ";	";
						LightCumulate += "	vec3 LumDir"+i+" = (LightPos" + i + ".xyz- WorldPos.xyz);	";
						LightCumulate += "	float Att"+i+" = pow(max((90.0-length(LumDir"+i+")) / 80.0,0.0), 2.0);	";
						LightCumulate += "	Lum += max(dot(normalize(LumDir"+i+"), WNorm), 0.0) * Att"+i+";	";
						LightCumulate += "	Spec += pow(max(dot(normalize(normalize(LumDir"+i+") + V), WNorm), 0.0), 10.0) * Att"+i+";	";
					}
					
					var passThroughVert = [
						"attribute vec3 vertex_position;",
						"attribute vec2 vertex_texCoord0;",
						"attribute vec3 vertex_normal;",
						"attribute vec4 vertex_tangent;",
						"uniform mat4 matrix_model;",
						"uniform mat4 matrix_viewProjection;",
						"uniform vec3 view_position;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Normal;",
						"varying vec4 Tangent;",
						"varying vec3 ViewDir;",
						"",
						"void main(void)",
						"{",
						"	vec4 positionW = matrix_model * vec4(vertex_position, 1.0);",
						"	WorldPos = positionW;",
						"	vUv0 = vertex_texCoord0;",
						"	Normal = (matrix_model * vec4(vertex_normal, 0.0)).xyz;",
						"	Tangent = vec4((matrix_model * vec4(vertex_tangent.xyz, 0.0)).xyz, vertex_tangent.w);",
						"	ViewDir = (view_position - positionW.xyz);",
						"	gl_Position = matrix_viewProjection * positionW;",
						"}"
					].join("\n");
					
					var colorFrag = [
						"precision mediump float;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Normal;",
						"varying vec4 Tangent;",
						"varying vec3 ViewDir;",
						"uniform sampler2D tex1;",
						"uniform sampler2D tex2;",
						"uniform sampler2D tex3;",
						"uniform sampler2D tex4;",
						"uniform sampler2D texN;",
						"uniform float Time;",
						"",
						LightDeclaration,
						"",
						"void main(void)",
						"{",
						"   vec3 NMap = normalize(texture2D(texN, vUv0).rgb * vec3(2.0) - vec3(1.0));",
						"	vec3 NN = normalize(Normal);",
						"	vec3 NT = normalize(Tangent.xyz);",
						"	vec3 NBN = -normalize(cross(NN, NT)) * Tangent.w;",
						"	vec3 WNorm = normalize(NMap.r * NT + NMap.g * NBN + NMap.b * NN);",
						"	vec3 V = normalize(ViewDir.xyz);",
						"	float Lum = 0.0;",
						"	float Spec = 0.0;",
						LightCumulate,
						"   vec4 diff = vec4(0.0);",
						"   diff.x = texture2D(tex1, vUv0).r;",
						"   diff.y = texture2D(tex2, vUv0).r;",
						"   diff.z = texture2D(tex3, vUv0).r;",
						"   diff.w = texture2D(tex4, vUv0).r;",						
						"   vec4 Factors = vec4(1.0)-clamp(abs(vec4(clamp(Lum,0.0,1.0)*3.0) - vec4(3.0, 2.0, 1.0, 0.0)), vec4(0.0), vec4(1.0));",
						"   float FinalLum = clamp((dot(Factors, diff)-0.5) * 10.0, 0.0, 1.0);",
						"	vec3 MatColor = mix(vec3(0.4, 0.29, 0.03), vec3(0.82, 0.78, 0.44), vec3(Lum));",
						//"	vec3 MatColor = vec3(1.0, 1.0, 1.0);",
						"   gl_FragColor = vec4(clamp(MatColor * vec3(FinalLum) + vec3(Spec), vec3(0.0), vec3(1.0)), 1.0);",
						//"   gl_FragColor = vec4(vec3(Spec), 1.0);",
						"}"
					].join("\n");
					
					var passThroughShader = new pc.gfx.Shader(pc.gfx.ShaderType.VERTEX, passThroughVert);
					var colorShader = new pc.gfx.Shader(pc.gfx.ShaderType.FRAGMENT, colorFrag);
					var program = new pc.gfx.Program(passThroughShader, colorShader);
					
					var passThroughVert2 = [
						"attribute vec3 vertex_position;",
						"attribute vec2 vertex_texCoord0;",
						"attribute vec3 vertex_normal;",
						"attribute vec4 vertex_tangent;",
						"uniform mat4 matrix_model;",
						"uniform mat4 matrix_viewProjection;",
						"uniform vec3 view_position;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Normal;",
						"varying vec4 Tangent;",
						"varying vec3 ViewDir;",
						"uniform float Time;",
						"",
						"void main(void)",
						"{",
						"	vec3 LocalPos = vertex_position;",
						"	float MotionWeight = clamp(1.0 - abs(LocalPos.x - 6.0) / 5.0, 0.0, 1.0);",
						"	vec3 LocalOffset = MotionWeight * sin(LocalPos*2.0 + vec3(Time * 3.0)) * 0.2;",
						"	vec4 positionW = matrix_model * vec4(LocalPos + LocalOffset, 1.0);",
						"	WorldPos = positionW;",
						"	vUv0 = vertex_texCoord0;",
						"	Normal = (matrix_model * vec4(vertex_normal, 0.0)).xyz;",
						"	Tangent = vec4((matrix_model * vec4(vertex_tangent.xyz, 0.0)).xyz, vertex_tangent.w);",
						"	ViewDir = (view_position - positionW.xyz);",
						"	gl_Position = matrix_viewProjection * positionW;",
						"}"
					].join("\n");
										
					var passThroughShader2 = new pc.gfx.Shader(pc.gfx.ShaderType.VERTEX, passThroughVert2);
					var program2 = new pc.gfx.Program(passThroughShader2, colorShader);
					
					var passThroughVert3 = [
						"attribute vec3 vertex_position;",
						"attribute vec2 vertex_texCoord0;",
						"attribute vec3 vertex_normal;",
						"attribute vec4 vertex_tangent;",
						"uniform mat4 matrix_model;",
						"uniform mat4 matrix_viewProjection;",
						"uniform vec3 view_position;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Normal;",
						"varying vec4 Tangent;",
						"varying vec3 ViewDir;",
						"",
						"void main(void)",
						"{",
						"	vec4 positionW = matrix_model * vec4(vertex_position, 1.0);",
						"	WorldPos = positionW;",
						"	vUv0 = vertex_texCoord0;",
						"	Normal = (matrix_model * vec4(vertex_normal, 0.0)).xyz;",
						"	Tangent = vec4((matrix_model * vec4(vertex_tangent.xyz, 0.0)).xyz, vertex_tangent.w);",
						"	ViewDir = (view_position - positionW.xyz);",
						"	gl_Position = matrix_viewProjection * positionW;",
						"}"
					].join("\n");
					
					var colorFrag3 = [
						"precision mediump float;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Normal;",
						"varying vec4 Tangent;",
						"varying vec3 ViewDir;",
						"uniform sampler2D tex1;",
						"uniform sampler2D tex2;",
						"uniform sampler2D tex3;",
						"uniform sampler2D tex4;",
						"uniform sampler2D texN;",
						"uniform float Time;",
						"",
						"void main(void)",
						"{",
						"   vec3 NMap1 = (texture2D(texN, vUv0 * 0.9 + vec2(0.1)*Time).rgb * vec3(2.0) - vec3(1.0));",
						"	vec2 uv2 = vUv0 + NMap1.rg * 0.15;",
						"	vec2 uv3 = vUv0 + NMap1.rg * 0.02 - vec2(-0.07, 0.03)*Time;",
						"	vec2 uv4 = vUv0 + NMap1.rg * 0.2 - vec2(-0.07, 0.03)*Time;",
						"   vec3 NMap3 = (texture2D(texN, uv4).rgb * vec3(2.0) - vec3(1.0));",
						"   vec3 NMap2 = normalize(texture2D(texN, uv2 - vec2(0.14, 0.31)*Time).rgb * 5.0 * vec3(2.0, 2.0, 1.0) - vec3(1.0, 1.0, 0.0));",
						"	float Lum = max(dot(vec3(0.0,0.0,0.5), NMap2), 0.0);",
						"	float Spec = max((dot(NMap3, NMap2) + 0.6) * 0.8, 0.0);",
						"   vec4 diff = vec4(0.0);",
						"   diff.x = texture2D(tex1, uv3).r;",
						"   diff.y = texture2D(tex2, uv3).r;",
						"   diff.z = texture2D(tex3, uv3).r;",
						"   diff.w = texture2D(tex4, uv3).r;",						
						"   vec4 Factors = vec4(1.0)-clamp(abs(vec4(clamp(Lum,0.0,1.0)*3.0) - vec4(3.0, 2.0, 1.0, 0.0)), vec4(0.0), vec4(1.0));",
						"   float FinalLum = clamp((dot(Factors, diff)-0.5) * 10.0, 0.0, 1.0);",
						"	vec3 MatColor = vec3(0.4, 0.29, 0.03);",
						"   gl_FragColor = vec4(clamp(MatColor * FinalLum + Spec*vec3(0.9, 0.6, 0.12)*(FinalLum*0.4+0.6), vec3(0.0), vec3(1.0)), 1.0);",
						//"   gl_FragColor = vec4(Lum, Lum, Lum, 1.0);",
						"}"
					].join("\n");
					
					var passThroughShader3 = new pc.gfx.Shader(pc.gfx.ShaderType.VERTEX, passThroughVert3);
					var colorShader3 = new pc.gfx.Shader(pc.gfx.ShaderType.FRAGMENT, colorFrag3);
					var program3 = new pc.gfx.Program(passThroughShader3, colorShader3);
					
					var passThroughVert4 = [
						"attribute vec3 vertex_position;",
						"attribute vec2 vertex_texCoord0;",
						"attribute vec3 vertex_normal;",
						"attribute vec4 vertex_tangent;",
						"uniform mat4 matrix_model;",
						"uniform mat4 matrix_viewProjection;",
						"uniform vec3 view_position;",
						"uniform float Time;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Color;",
						"",
						"void main(void)",
						"{",
						"	vec3 LP = vec3(floor(vertex_position.x + 0.1), floor(vertex_position.y + 0.1), 3.0);",
						"	vec4 DD = vec4(fract(vertex_position.x) * 9.0, fract(vertex_position.y) * 16.0, 0.0, 0.0);",
						"	float Num = LP.x + 10.0 * LP.y;",
						"	vec3 motion = normalize(sin(LP * vec3(1.06791, 1.03423, 1.0495) * sin(Num * 0.12) + Time * 0.3 * vec3(1.0, 2.4, 3.4)));",
						"	vec4 positionW = matrix_model * vec4(motion, 1.0);",
						"	vec4 Screen = matrix_viewProjection * positionW + DD * 1.0;",				
						"	WorldPos = Screen;",
						"	vUv0 = vertex_texCoord0;",
						"	Color = DD.xyz;",
						"	gl_Position = Screen;",
						"}"
					].join("\n");
					
					var colorFrag4 = [
						"precision mediump float;",
						"",
						"varying vec4 WorldPos;",
						"varying vec2 vUv0;",
						"varying vec3 Color;",
						"uniform sampler2D tex1;",
						"uniform float Time;",
						"",
						"void main(void)",
						"{",
						"   vec3 N1 = texture2D(tex1, fract(vUv0)).rgb;",
						"	gl_FragColor = vec4(N1 * vec3(0.97, 0.92, 0.65), 1.0);",
						"}"
					].join("\n");
					
					var passThroughShader4 = new pc.gfx.Shader(pc.gfx.ShaderType.VERTEX, passThroughVert4);
					var colorShader4 = new pc.gfx.Shader(pc.gfx.ShaderType.FRAGMENT, colorFrag4);
					var program4 = new pc.gfx.Program(passThroughShader4, colorShader4);
										
					this.createWorld();
					
					pc.scene.Scene.NPRProgram = program;
					pc.scene.Scene.NPRProgram2 = program2;
					pc.scene.Scene.NPRProgram3 = program3;
					pc.scene.Scene.NPRProgram4 = program4;
				}
			}
        },
				
		createWorld: function () {
			
			sizeX = 10;
			sizeY = 10;
			world = new Array(sizeX*sizeY);
			this.entity.sizeX = sizeX;
			this.entity.sizeY = sizeY;
			this.entity.world = world;
			
			// Everything is plain
			for(var i=0; i<(sizeX*sizeY); ++i)
			{
				world[i] = 1;
			}
				
			for(var x=0; x<sizeX; ++x)
			{
				for(var y=0; y<sizeY; ++y)
				{
					if(x == 0 || x == (sizeX-1) || y == 0 || y == (sizeY-1))
					{	
						world[x + y*sizeX] = 1;
					}
					else
					{					
						var tmp = random();
						world[x + y*sizeX] = (tmp > 0.7) ? 1 : 0;
					}
				}
			}
			/*
			for(var x=0; x<sizeX; ++x)
			{
				for(var y=0; y<sizeY; ++y)
				{
					world[x + y*sizeX] = (y%2 == 0) ? 1 : 0;
				}
			}*/
			
			var startX = sizeX/2;
			var startY = sizeY/2;
			
			world[startX + startY * sizeX] = 0;
						
			this.MainCamera = this.entity._parent.findByName('Camera');
			this.MainCamera.setPosition(startX * 50, 25, startY * 50);
		
			// search proper objects to clone :
			var wallToClone = this.entity._parent.findByName('Wall01');
			var pillar = this.entity._parent.findByName('Pillar');
			var ground = this.entity._parent.findByName('Ground');
			var thing = this.entity._parent.findByName('Thing');
			var light01 = this.entity._parent.findByName('Light01');
			var part = this.entity._parent.findByName('Part');
			
			for(var x=1; x<sizeX; ++x)
			{
				for(var y=1; y<sizeY; ++y)
				{
					var c0 = world[x + y*sizeX];
					var cX = world[x - 1 + y*sizeX];
					var cY = world[x + (y-1)*sizeX];
					var cXY = world[x - 1 + (y-1)*sizeX];
					
					var uX = x<(sizeX-1) ? 1 : 0;
					var uY = y<(sizeY-1) ? 1 : 0;
					var cX2 = world[x + uX + y*sizeX];
					var cY2 = world[x + (y+uY)*sizeX];
					
					if(!c0)
					{
						var newEntity = copyEntity(ground);
						context.root.addChild(newEntity);
						newEntity.setPosition(x * 50 - 25, 0, y * 50 + 25);		

						if(random() > 0.7)
						{
							var newEntity = copyEntity(light01);
							context.root.addChild(newEntity);
							newEntity.setPosition(x * 50, 35, y * 50);	
							
							//var newEntity = copyEntity(part);
							//context.root.addChild(newEntity);
							//newEntity.setPosition(x * 50, 35, y * 50);	
						}
						
						
						if(cX && cX2)
						{
							//if(random() > 0.7)
							{
								var newEntity = copyEntity(thing);
								context.root.addChild(newEntity);
								newEntity.setPosition(x * 50 + 25, 0, y * 50 - 25);
								newEntity.rotate(0, 180, 0);
							}
						}
						
						if(cY && cY2)
						{
							//if(random() > 0.7)
							{
								var newEntity = copyEntity(thing);
								context.root.addChild(newEntity);
								newEntity.setPosition(x * 50 - 25, 0, y * 50 + 25);
								newEntity.rotate(0, 90, 0);
							}
						}
					}
					
					if(c0 != cX)
					{
						var newEntity = copyEntity(wallToClone);
						context.root.addChild(newEntity);
						var sign = (cX == 0)?1:-1;
						
						newEntity.setPosition(x * 50 - 25, 0, y * 50 + 25 * sign);
						newEntity.rotate(0, 90 * sign , 0);
					}
					if(c0 != cY)
					{
						var newEntity = copyEntity(wallToClone);
						context.root.addChild(newEntity);
						var sign = (cY == 0)?-1:1;
						
						newEntity.setPosition(x * 50 + 25 * sign, 0, y * 50 - 25);
						newEntity.rotate(0, 90 + 90 * sign , 0);
					}
					if((c0 != cX) != (cY != cXY))
					{
						var newEntity = copyEntity(pillar);
						context.root.addChild(newEntity);
						newEntity.setPosition(x * 50 - 25, 0, y * 50 - 25);
					}
				}
			}
		}
    };
 
    return GlobalScript;
});